﻿/*
 *  TOPPERS/ECNL Prototyping tool
 *
 *  Copyright (C) 2017 Cores Co., Ltd. Japan
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id$
 */
using System;
using System.Linq;
using System.Collections.Generic;
using Bridge;
using Bridge.Linq;
using Bridge.Text.RegularExpressions;
using System.Collections;

namespace WebMrbc
{
	public interface IModel
	{
		string Identifier { get; }
	}

	public class Collection<T> : IEnumerable<T> where T : IModel
	{
		T[] List = new T[0];

		public Collection()
		{
		}

		public T LastModel { get { int i = List.Length; if (i == 0) return default(T); return List[i - 1]; } }
		public int Length { get { return List.Length; } }

		public EventHandler OnAdd;
		public EventHandler OnRemove;
		public EventHandler OnReset;
		public EventHandler OnChange;

		internal void Add(T item)
		{
			List.Push(item);

			if (OnAdd != null)
				OnAdd(this, EventArgs.Empty);
		}

		internal void Remove(T item)
		{
			var i = List.IndexOf(item);
			if (i >= 0)
				List.Splice(i, 1);

			if (OnRemove != null)
				OnRemove(this, EventArgs.Empty);
		}

		internal void Reset()
		{
			List.Splice(0, List.Length);

			if (OnReset != null)
				OnReset(this, EventArgs.Empty);
		}

		internal void Reset(IEnumerable<T> n)
		{
			List.Splice(0, List.Length);
			List = (T[])List.Concat(n);

			if (OnReset != null)
				OnReset(this, EventArgs.Empty);
		}

		internal T At(int index)
		{
			return List[index];
		}

		internal string UniqueName(string identifier)
		{
			var prefix = "";
			if ((identifier[0] >= 'a') && (identifier[0] <= 'z'))
				prefix = identifier[0].ToString().ToUpper();
			else if ((identifier[0] < 'A') || (identifier[0] > 'Z'))
				prefix = "C";

			var max = 0;
			var n = new Regex("^" + identifier + "([0-9]+)$");
			foreach (var c in List) {
				var m = c.Identifier.Match(n);
				if (m != null)
					max = System.Math.Max(max, Script.ParseInt(m[0]));
				break;
			}
			if (max == 0)
				return prefix + identifier;
			else
				return prefix + identifier + max;
		}

		internal T FindWhere(object p)
		{
			foreach (var c in List) {
				if (Script.Write<bool>("(function(c) { for (var i in p) if (p[i] !== c[i]) return false; return true; })(c)"))
					return c;
			}
			return default(T);
		}

		public IEnumerator<T> GetEnumerator()
		{
			return (dynamic)List.GetEnumerator();
		}

		IEnumerator IEnumerable.GetEnumerator()
		{
			return List.GetEnumerator();
		}
	}
}
